@@ -534,7 +534,7 @@ class Box(WorldObj):
 
     def toggle(self, env, pos):
         # Replace the box by its contents
-        env.grid.set(*pos, self.contains)
+        env.grid.set(pos[0], pos[1], self.contains)
         return True
 
 
@@ -696,7 +696,7 @@ class Grid:
 
         return img
 
-    def render(self, tile_size, agent_pos=None, agent_dir=None, highlight_mask=None):
+    def render(self, tile_size, agent_pos, agent_dir=None, highlight_mask=None):
         """
         Render this grid at a given scale
         :param r: target renderer object
@@ -825,10 +825,7 @@ class MiniGridEnv(gym.Env):
     """
 
     metadata = {
-        # Deprecated: use 'render_modes' instead
-        "render.modes": ["human", "rgb_array"],
-        "video.frames_per_second": 10,  # Deprecated: use 'render_fps' instead
-        "render_modes": ["human", "rgb_array", "single_rgb_array"],
+        "render_modes": ["human", "rgb_array"],
         "render_fps": 10,
     }
 
@@ -838,7 +835,6 @@ class MiniGridEnv(gym.Env):
         left = 0
         right = 1
         forward = 2
-
         # Pick up an object
         pickup = 3
         # Drop an object
@@ -858,11 +854,11 @@ class MiniGridEnv(gym.Env):
         max_steps: int = 100,
         see_through_walls: bool = False,
         agent_view_size: int = 7,
+        render_mode: Optional[str] = None,
         highlight: bool = True,
         tile_size: int = TILE_PIXELS,
-        **kwargs,
+        agent_pov: bool = False,
     ):
-
         # Initialize mission
         self.mission = mission_space.sample()
 
@@ -914,21 +910,32 @@ class MiniGridEnv(gym.Env):
         self.agent_pos: np.ndarray = None
         self.agent_dir: int = None
 
-        # Initialize the state
-        self.reset()
+        # Current grid and mission and carryinh
+        self.grid = Grid(width, height)
+        self.carrying = None
 
-    def reset(self, *, seed=None, return_info=False, options=None):
+        # Rendering attributes
+        self.render_mode = render_mode
+        self.highlight = highlight
+        self.tile_size = tile_size
+        self.agent_pov = agent_pov
+
+    def reset(self, *, seed=None, options=None):
         super().reset(seed=seed)
-        # Current position and direction of the agent
-        self.agent_pos = None
-        self.agent_dir = None
+
+        # Reinitialize episode-specific variables
+        self.agent_pos = (-1, -1)
+        self.agent_dir = -1
 
         # Generate a new random grid at the start of each episode
         self._gen_grid(self.width, self.height)
 
         # These fields should be defined by _gen_grid
-        assert self.agent_pos is not None
-        assert self.agent_dir is not None
+        assert (
+            self.agent_pos >= (0, 0)
+            if isinstance(self.agent_pos, tuple)
+            else all(self.agent_pos >= 0) and self.agent_dir >= 0
+        )
 
         # Check that the agent doesn't overlap with an object
         start_cell = self.grid.get(*self.agent_pos)
@@ -940,13 +947,13 @@ class MiniGridEnv(gym.Env):
         # Step count since episode start
         self.step_count = 0
 
+        if self.render_mode == "human":
+            self.render()
+
         # Return first observation
         obs = self.gen_obs()
 
-        if not return_info:
-            return obs
-        else:
-            return obs, {}
+        return obs, {}
 
     def hash(self, size=16):
         """Compute a hash that uniquely identifies the current state of the environment.
@@ -1126,6 +1133,8 @@ class MiniGridEnv(gym.Env):
                 )
             )
 
+            pos = tuple(pos)
+
             # Don't place the object on top of another object
             if self.grid.get(*pos) is not None:
                 continue
@@ -1140,7 +1149,7 @@ class MiniGridEnv(gym.Env):
 
             break
 
-        self.grid.set(*pos, obj)
+        self.grid.set(pos[0], pos[1], obj)
 
         if obj is not None:
             obj.init_pos = pos
@@ -1162,7 +1171,7 @@ class MiniGridEnv(gym.Env):
         Set the agent's starting point at an empty position in the grid
         """
 
-        self.agent_pos = None
+        self.agent_pos = (-1, -1)
         pos = self.place_obj(None, top, size, max_tries=max_tries)
         self.agent_pos = pos
 
@@ -1292,13 +1301,16 @@ class MiniGridEnv(gym.Env):
         obs_cell = obs_grid.get(vx, vy)
         world_cell = self.grid.get(x, y)
 
+        assert world_cell is not None
+
         return obs_cell is not None and obs_cell.type == world_cell.type
 
     def step(self, action):
         self.step_count += 1
 
         reward = 0
-        done = False
+        terminated = False
+        truncated = False
 
         # Get the position in front of the agent
         fwd_pos = self.front_pos
@@ -1319,24 +1331,25 @@ class MiniGridEnv(gym.Env):
         # Move forward
         elif action == self.actions.forward:
             if fwd_cell is None or fwd_cell.can_overlap():
-                self.agent_pos = fwd_pos
+                self.agent_pos = tuple(fwd_pos)
             if fwd_cell is not None and fwd_cell.type == "goal":
-                done = True
+                terminated = True
                 reward = self._reward()
             if fwd_cell is not None and fwd_cell.type == "lava":
-                done = True
+                terminated = True
+
         # Pick up an object
         elif action == self.actions.pickup:
             if fwd_cell and fwd_cell.can_pickup():
                 if self.carrying is None:
                     self.carrying = fwd_cell
                     self.carrying.cur_pos = np.array([-1, -1])
-                    self.grid.set(*fwd_pos, None)
+                    self.grid.set(fwd_pos[0], fwd_pos[1], None)
 
         # Drop an object
         elif action == self.actions.drop:
             if not fwd_cell and self.carrying:
-                self.grid.set(*fwd_pos, self.carrying)
+                self.grid.set(fwd_pos[0], fwd_pos[1], self.carrying)
                 self.carrying.cur_pos = fwd_pos
                 self.carrying = None
 
@@ -1350,14 +1363,17 @@ class MiniGridEnv(gym.Env):
             pass
 
         else:
-            assert False, "unknown action"
+            raise ValueError(f"Unknown action: {action}")
 
         if self.step_count >= self.max_steps:
-            done = True
+            truncated = True
+
+        if self.render_mode == "human":
+            self.render()
 
         obs = self.gen_obs()
 
-        return obs, reward, done, {}
+        return obs, reward, terminated, truncated, {}
 
     def gen_obs_grid(self, agent_view_size=None):
         """
@@ -1406,10 +1422,6 @@ class MiniGridEnv(gym.Env):
         # Encode the partially observable view into a numpy array
         image = grid.encode(vis_mask)
 
-        assert hasattr(
-            self, "mission"
-        ), "environments must define a textual mission string"
-
         # Observations are dictionaries containing:
         # - an image (partially observable view of the environment)
         # - the agent's direction/orientation (acting as a compass)
@@ -1418,12 +1430,11 @@ class MiniGridEnv(gym.Env):
 
         return obs
 
-    def get_obs_render(self, obs, tile_size=TILE_PIXELS // 2):
+    def get_pov_render(self, tile_size):
         """
-        Render an agent observation for visualization
+        Render an agent's POV observation for visualization
         """
-
-        grid, vis_mask = Grid.decode(obs)
+        grid, vis_mask = self.gen_obs_grid()
 
         # Render the whole grid
         img = grid.render(
@@ -1435,15 +1446,10 @@ class MiniGridEnv(gym.Env):
 
         return img
 
-    def render(self, mode="human", highlight=True, tile_size=TILE_PIXELS):
-        assert mode in self.metadata["render_modes"]
+    def get_full_render(self, highlight, tile_size):
         """
-        Render the whole-grid human view
+        Render a non-paratial observation for visualization
         """
-        if mode == "human" and not self.window:
-            self.window = Window("gym_minigrid")
-            self.window.show(block=False)
-
         # Compute which cells are visible to the agent
         _, vis_mask = self.gen_obs_grid()
 
@@ -1486,10 +1492,44 @@ class MiniGridEnv(gym.Env):
             highlight_mask=highlight_mask if highlight else None,
         )
 
-        if mode == "human":
+        return img
+
+    def get_frame(
+        self,
+        highlight: bool = True,
+        tile_size: int = TILE_PIXELS,
+        agent_pov: bool = False,
+    ):
+        """Returns an RGB image corresponding to the whole environment or the agent's point of view.
+
+        Args:
+
+            highlight (bool): If true, the agent's field of view or point of view is highlighted with a lighter gray color.
+            tile_size (int): How many pixels will form a tile from the NxM grid.
+            agent_pov (bool): If true, the rendered frame will only contain the point of view of the agent.
+
+        Returns:
+
+            frame (np.ndarray): A frame of type numpy.ndarray with shape (x, y, 3) representing RGB values for the x-by-y pixel image.
+
+        """
+
+        if agent_pov:
+            return self.get_pov_render(tile_size)
+        else:
+            return self.get_full_render(highlight, tile_size)
+
+    def render(self):
+
+        img = self.get_frame(self.highlight, self.tile_size, self.agent_pov)
+
+        if self.render_mode == "human":
+            if self.window is None:
+                self.window = Window("gym_minigrid")
+                self.window.show(block=False)
             self.window.set_caption(self.mission)
             self.window.show_img(img)
-        else:
+        elif self.render_mode == "rgb_array":
             return img
 
     def close(self):
